define([ "app", "text!js/advanced/prerender.html", "js/advanced/advancedView" ], function(app, Template, View) {
	var bindings = [ {
		element : '#salva_avanzate',
		event : 'click',
		handler : setAdvanced
	}, {
		element : 'select[name="dhcp"]',
		event : 'change',
		handler : changeDHCP
	} ];

	function init() {
		app.utils.Log("INIT advancedController");

		app.f7.showIndicator();

		var compiled_template = Template7.compile(Template);
		var html = compiled_template({});

		app.mainView.router.loadPage({
			content : html
		});

		loadAdvanced();

		app.utils.googleAnalytics('send', 'pageview', {
			'page' : 'advanced'
		});
	}

	function loadAdvanced() {
		app.utils.ajaxRequest("GET", app.utils.GetRisorsa("status_advanced"), {
			callback : function(data) {
				if (data.success == true && data.net != undefined) {
					showAdvanced({
						net : data.net
					});
				} else {
					showAdvanced({});
					app.f7.addNotification({
						title : Template7.global.lbl.ERROR + ": " + data.error,
						hold : 2000
					});
				}
			},
			callback_error : function(jqXHR, textStatus, errorThrown) {
				showAdvanced({});
				app.f7.addNotification({
					title : Template7.global.lbl.ERROR + ": " + jqXHR.statusText,
					hold : 2000
				});
			},
			timeout : 10000,
			skipNotification : true,
			extraData : "cloud"
		});
	}

	function showAdvanced(data) {
		View.render({
			bindings : bindings,
			model : data
		});

		// Mask indirizzi IP
		$('.ip').mask('099.099.099.099');

		// Submit form
		$('#advanced').validate({
			rules : {
				ip : {
					required : true
				},
				sub : {
					required : true
				},
				gw : {
					required : true
				}
			},
			messages : {},
			submitHandler : function(form) {
				setAdvanced();
				return false;
			}
		});

		app.f7.hideIndicator();
	}

	function setAdvanced() {
		var dhcp = app.$$('select[name="dhcp"]').val(), ip = app.$$('input[name="ip"]').val(), sub = app.$$('input[name="sub"]').val(), gw = app.$$('input[name="gw"]').val();

		app.utils.ajaxRequest("POST", app.utils.GetRisorsa("network_set_advanced"), {
			callback : function(data) {
				if (data.success == true) {
					// salvo il nuovo IP del device
					var cur_dev = app.utils.getCurrentDevice();
					cur_dev.setIP(ip);
					cur_dev.save(app);
					
					// moreno.trainini, 18/05/2016
					// sovrascrivo il current device con quello con IP modificato
					app.utils.setCurrentDevice(cur_dev);
					
					app.f7.showPreloader(Template7.global.lbl.CONFIGURATION_IN_PROGRESS);
					setTimeout(function() {
						app.utils.checkSetup(app);
					}, 10000);
				} else {
					app.f7.addNotification({
						title : Template7.global.lbl.ERROR + ": " + data.error,
						hold : 2000
					});
				}
			},
			callback_error : function(jqXHR, text, status) {
			},
			data : {
				pwd : "-1",
				essid : "-1",
				dhcp : dhcp,
				ip : ip,
				sub : sub,
				gw : gw
			}
		});
	}

	function changeDHCP() {
		var dhcp = app.$$('select[name="dhcp"]').val();
		$('input[name="ip"]').prop('disabled', (dhcp == 1) ? true : false);
		$('input[name="sub"]').prop('disabled', (dhcp == 1) ? true : false);
		$('input[name="gw"]').prop('disabled', (dhcp == 1) ? true : false);
	}

	return {
		init : init
	};
});