define([ "app", "text!js/calendar-period-edit/prerender.html", "js/calendar-period-edit/calendar-period-editView" ], function(app, Template, View) {
	var bindings = [ {
		element : '#inizio',
		event : 'change',
		handler : changeTime
	}, {
		element : '#fine',
		event : 'change',
		handler : changeTime
	}, {
		element : '.save-period',
		event : 'click',
		handler : savePeriod
	} ];

	var periodi = "", indice = "", giorno = "", programmazione = "";

	function init(parameters) {
		app.f7.showIndicator();

		app.utils.Log("INIT calendar-period-editController");

		giorno = parameters.giorno;
		periodi = parameters.periodi;
		indice = parameters.indice;
		programmazione = parameters.programmazione;

		var titolo = "";
		if (indice != undefined) {
			// Modifica periodo
			titolo = Template7.global.lbl.EDIT_PERIOD;
		} else {
			// Nuovo periodo
			titolo = Template7.global.lbl.ADD_PERIOD;
		}

		var compiled_template = Template7.compile(Template);
		var context = {
			titolo : titolo
		};
		var html = compiled_template(context);

		app.mainView.router.loadPage({
			content : html
		});

		setTimeout(function() {
			loadPickers();

			app.utils.googleAnalytics('send', 'pageview', {
				'page' : 'calendar-period-edit'
			});
		}, 500);
	}

	function getTimes(parameters) {
		if (parameters == undefined) {
			parameters = {};
		}
		if (parameters.from == undefined) {
			parameters.from = 0;
		}
		if (parameters.to == undefined) {
			parameters.to = 48;
		}
		var times = [];
		for (i = parameters.from; i <= parameters.to; i++) {
			times.push({
				value : i,
				cls : (programmazione.charAt(i + (48 * giorno)) == 1) ? "gray" : "",
				label : ('0' + Math.floor(i / 2)).slice(-2) + ":" + ((i % 2) == 0 ? "00" : "30")
			});
		}
		return times;
	}

	function convertTimeToInt(time) {
		var str = time.split(":"), val = 0;

		val = parseInt(str[0]) * 2 + ((str[1] == "30") ? 1 : 0);
		return val;
	}

	function loadPickers() {
		var inizio = "";
		if (indice != undefined) {
			fine = periodi[indice].fine;
			inizio = periodi[indice].inizio;
		} else {
			inizio = "00:00";
			fine = "24:00";
		}
		var times_inizio = getTimes({
			to : 47
		});
		var times_fine = getTimes({
			from : convertTimeToInt(inizio) + 1
		});

		showPickers(times_inizio, times_fine);
	}

	function showPickers(times_inizio, times_fine) {
		var inizio = "", fine = "";
		if (indice != undefined) {
			inizio = periodi[indice].inizio;
			fine = periodi[indice].fine;
		} else {
			inizio = "00:00";
			fine = "24:00";
		}

		View.render({
			bindings : bindings,
			model : {
				times : {
					inizio : times_inizio,
					fine : times_fine
				},
				inizio : {
					value : convertTimeToInt(inizio),
					label : inizio
				},
				fine : {
					value : convertTimeToInt(fine),
					label : fine
				}
			}
		});
		app.f7.hideIndicator();
	}

	function changeTime() {
		if ($(this).attr("id") == "inizio") {
			var actual_fine = parseInt($('#fine').val()),
				new_val = parseInt($(this).val());
			if (actual_fine <= new_val) {
				var new_val = new_val + 1;
				$('#fine').val(new_val);
				$('#fine').parent().find('.item-after').html(('0' + Math.floor(new_val / 2)).slice(-2) + ":" + ((new_val % 2) == 0 ? "00" : "30"));
			}
			var new_times = getTimes({
				from : parseInt($(this).val()) + 1
			}), str = "";
			new_times.forEach(function(t) {
				var selected = "";
				if (t.value == $('#fine').val()) {
					selected = "selected";
				}
				str += '<option data-option-color="' + t.cls + '" value="' + t.value + '"' + selected + '>' + t.label + '</option>';
			});
			$('#fine').html(str);
		} else {
			var new_times = getTimes({
				to : 47
			}), str = "";
			new_times.forEach(function(t) {
				var selected = "";
				if (t.value == $('#inizio').val()) {
					selected = "selected";
				}
				str += '<option data-option-color="' + t.cls + '" value="' + t.value + '"' + selected + '>' + t.label + '</option>';
			});
			$('#inizio').html(str);
		}
	}

	function savePeriod() {
		var inizio = $('#inizio').val(), fine = $('#fine').val(), other_period = false, old_inizio = $('#old_inizio').val(), old_fine = $('#old_fine').val();

		inizio = parseInt(inizio) + (48 * giorno), fine = parseInt(fine) + (48 * giorno);
		old_inizio = parseInt(old_inizio) + (48 * giorno), old_fine = parseInt(old_fine) + (48 * giorno);

		if (indice == undefined) {
			for (i = inizio; i < fine; i++) {
				programmazione = app.utils.replaceAt(programmazione, i, '1');
			}
			setCalendar(programmazione);
		} else {
			// Elimino la sequenza vecchia (tutti 0)
			for (i = old_inizio; i < old_fine; i++) {
				programmazione = app.utils.replaceAt(programmazione, i, '0');
			}
			// Creo la sequenza nuova
			for (i = inizio; i < fine; i++) {
				programmazione = app.utils.replaceAt(programmazione, i, '1');
			}
			setCalendar(programmazione);
		}
	}

	function setCalendar(programmazione) {
		app.utils.ajaxRequest("POST", app.utils.GetRisorsa("calendar"), {
			callback : function(data) {
				if (data.success == true) {
					app.utils.setLocalConfig("programmazione", programmazione);
					app.f7.addNotification({
						title : Template7.global.lbl.SET_CALENDAR,
						hold : 2000
					});
					app.mainView.router.back();
				} else {
					app.f7.addNotification({
						title : Template7.global.lbl.ERROR + ": " + data.error,
						hold : 2000
					});
				}
			},
			data : {
				calendar : programmazione
			}
		});
	}

	return {
		init : init
	};
});