define([ "app", "js/calendar/calendarController", "text!js/calendar-period/prerender.html", "js/calendar-period/calendar-periodView" ], function(app, CalendarRef, Template, View) {
	var bindings = [ {
		element : '.add-period',
		event : 'click',
		handler : addPeriod
	}, {
		element : '.edit-period',
		event : 'click',
		handler : editPeriod
	}, {
		element : '.delete-period',
		event : 'click',
		handler : deletePeriod
	} ];

	var giorno_val = "", giorno_name = "", programmazione_oraria = {}, programmazione;

	function init(parameters) {
		app.f7.showIndicator();

		app.utils.Log("INIT calendar-periodController");

		giorno = parameters.giorno;
		giorno_val = giorno;
		giorno_name = Template7.global.lbl["DAY_" + giorno_val];

		var compiled_template = Template7.compile(Template);
		var context = {
			model : giorno_name
		};
		var html = compiled_template(context);

		app.mainView.router.loadPage({
			content : html
		});

		setTimeout(function() {
			loadPeriods();

			app.utils.googleAnalytics('send', 'pageview', {
				'page' : 'calendar-period'
			});
		}, 500);
	}

	function convertCalendarToObject(programmazione) {
		var str = {}, contatore = 0;
		programmazione_oraria.periodi = new Array();

		for (i = 0; i < 7; i++) {
			from = -1;
			for (k = 0; k < 24 * 2; k++) {
				if (programmazione.charAt(contatore) == 1) {
					if (k < 47) { // sono arrivato alla fine
						if (from == -1) {
							from = k;
						}
					} else {
						if (from == -1) {
							// In caso abbia l'ultima mezz'ora mi salvo il
							// puntatore iniziale
							from = k;
						}
						programmazione_oraria.periodi.push({
							giorno : i,
							contatore : programmazione_oraria.periodi.length,
							inizio : ('0' + Math.floor(from / 2)).slice(-2) + ":" + ((from % 2) == 0 ? "00" : "30"),
							fine : ('0' + Math.floor((k + 1) / 2)).slice(-2) + ":" + (((k + 1) % 2) == 0 ? "00" : "30"),
						});
						from = -1;
					}
				} else {
					if (from != -1) {
						programmazione_oraria.periodi.push({
							giorno : i,
							contatore : programmazione_oraria.periodi.length,
							inizio : ('0' + Math.floor(from / 2)).slice(-2) + ":" + ((from % 2) == 0 ? "00" : "30"),
							fine : ('0' + Math.floor(k / 2)).slice(-2) + ":" + ((k % 2) == 0 ? "00" : "30"),
						});
						from = -1;
					}
				}
				contatore++;
			}
		}

		return programmazione_oraria;
	}

	function refreshPeriods() {
		loadPeriods();
	}

	function loadPeriods() {
		var program = app.utils.getLocalConfig("programmazione");
		if (program != undefined && program.length == 336) {
			programmazione = program;
			showPeriods(convertCalendarToObject(program));
		} else {
			app.utils.ajaxRequest("GET", app.utils.GetRisorsa("calendar"), {
				callback : function(data) {
					if (data.success == true) {
						program = data.RESULT;
						programmazione = program;

						CalendarRef.splitCalendarPerDay(program);
						showPeriods(convertCalendarToObject(program));
					} else {
						app.f7.addNotification({
							title : Template7.global.lbl.ERROR + ": " + data.error,
							hold : 2000
						});
					}
				}
			});
		}
	}

	function showPeriods(programmazione_oraria) {
		View.render({
			bindings : bindings,
			model : {
				programmazione : programmazione_oraria,
				giorno : giorno_val
			}
		});
		app.f7.hideIndicator();
	}

	function addPeriod() {
		app.router.load('calendar-period-edit', {
			periodi : programmazione_oraria.periodi,
			programmazione : programmazione,
			giorno : giorno_val
		});
	}

	function editPeriod() {
		app.router.load('calendar-period-edit', {
			periodi : programmazione_oraria.periodi,
			programmazione : programmazione,
			giorno : giorno_val,
			indice : $(this).attr('k_index')
		});
	}

	function convertTimeToInt(time) {
		var str = time.split(":"), val = 0;

		val = parseInt(str[0]) * 2 + ((str[1] == "30") ? 1 : 0);
		return val;
	}

	function setCalendar(programmazione) {
		app.utils.ajaxRequest("POST", app.utils.GetRisorsa("calendar"), {
			callback : function(data) {
				if (data.success == true) {
					app.utils.setLocalConfig("programmazione", programmazione);

					app.f7.addNotification({
						title : Template7.global.lbl.SET_CALENDAR,
						hold : 2000
					});

					programmazione_oraria = convertCalendarToObject(programmazione);
					loadPeriods(programmazione_oraria);
				} else {
					app.f7.addNotification({
						title : Template7.global.lbl.ERROR + ": " + data.error,
						hold : 2000
					});
				}
			},
			data : {
				calendar : programmazione
			}
		});
	}

	function deletePeriod() {
		var index = $(this).attr('k_index');
		app.f7.confirm(Template7.global.lbl.SURE_TO_DELETE_PERIOD, function() {
			var periodo = programmazione_oraria.periodi[index], inizio = convertTimeToInt(periodo.inizio) + (48 * periodo.giorno), fine = convertTimeToInt(periodo.fine) + (48 * periodo.giorno);

			for (i = inizio; i <= fine; i++) {
				programmazione = app.utils.replaceAt(programmazione, i, '0');
			}
			setCalendar(programmazione);
		}, function() {
		});
	}

	return {
		init : init,
		refreshPeriods : refreshPeriods
	};
});