define([ "app", "text!js/calendar/prerender.html", "js/calendar/calendarView" ], function(app, Template, View) {
	var bindings = [ {
		element : '.edit-calendar',
		event : 'click',
		handler : showDetailedPeriod
	}, {
		element : '.duplicate-calendar',
		event : 'click',
		handler : allowCopyCalendar
	}, {
		element : '.copy-calendar',
		event : 'click',
		handler : copyCalendar
	} ];

	var programmazione = {};

	function init() {
		app.f7.showIndicator();

		app.utils.Log("INIT calendarController");

		// Pre render
		var compiled_template = Template7.compile(Template);
		var html = compiled_template({});

		app.mainView.router.loadContent(html);
		
		setTimeout(function() {
			loadCalendar();

			app.utils.googleAnalytics('send', 'pageview', {
				'page' : 'calendar'
			});
		}, 500);
	}

	function refreshCalendar() {		
		// Nascondo il bottone di salvataggio
		$('.copy-calendar').attr('k_day', '').addClass('hidden');
		
		// Nascondo il bottone di copia se non c'è almeno un giorno checkato
		$('.copy-calendar').addClass('hidden');
		
		// Tornando i dietro non richiedo nuovamente i dati
		loadCalendar(false);
	}

	function loadCalendar(refresh) {
		if (refresh == undefined) {
			refresh = true;
		}

		var program = app.utils.getLocalConfig("programmazione");
		if (!refresh && program != undefined && program.length == 336) {
			programmazione.string = program;
			programmazione.scheduling = splitCalendarPerDay(program);
			showCalendar(programmazione.scheduling);
		} else {
			// Richiedo dati solamente al nuovo accesso
			app.utils.ajaxRequest("GET", app.utils.GetRisorsa("calendar"), {
				callback : function(data) {
					if (data.success == true && data.RESULT != undefined) {
						// Salvo in localStorage la programmazione del
						// calendario
						app.utils.setLocalConfig("programmazione", data.RESULT);

						programmazione.string = data.RESULT;
						programmazione.scheduling = splitCalendarPerDay(data.RESULT);
						showCalendar(programmazione.scheduling);
					} else {
						app.f7.addNotification({
							title : Template7.global.lbl.ERROR + ": " + data.error,
							hold : 2000
						});
					}
				}
			});
		}
	}

	function showCalendar(data) {		
		// Post render
		View.render({
			bindings : bindings,
			model : data
		});

		app.f7.hideIndicator();
	}
	
	function getDayOfWeek() {
		var d = new Date(), day_of_week = d.getDay(), days = [ 6, 0, 1, 2, 3, 4, 5 ];
		return days[day_of_week];
	}

	function splitCalendarPerDay(program) {
		var calendar = new Array(), contatore = 0;
		for (i = 0; i < 7; i++) {
			var str = "DAY_" + i;
			calendar[i] = {
				giorno : i,
				etichetta : Template7.global.lbl[str],
				day_of_week : getDayOfWeek(),
				valori : new Array()
			};
			for (k = 0; k < 24 * 2; k++) {
				calendar[i].valori.push(program.charAt(contatore));
				contatore++;
			}
		}
		return calendar;
	}

	function showDetailedPeriod() {
		var giorno = $(this).attr("k_day");

		app.router.load('calendar-period', {
			giorno : giorno
		});
	}

	function allowCopyCalendar() {
		if ($(this).children("i").hasClass('icon-check-0')) {
			// Lo checko
			$(this).children("i").removeClass('icon-check-0').addClass('icon-check-1');

			// Visualizzo il bottone di copia solo se c'è almeno un giorno
			// checkato
			if ($('.icon-check-1').length > 0) {
				$('.copy-calendar').removeClass('hidden');
			}
		} else if ($(this).children("i").hasClass('icon-check-1')) {
			// Lo de-checko
			$(this).children("i").removeClass('icon-check-1').addClass('icon-check-0');

			// Nascondo il bottone di copia se non c'è almeno un giorno checkato
			if ($('.icon-check-1').length <= 0) {
				$('.copy-calendar').addClass('hidden');
			}
		} else if ($(this).children("i").hasClass("icon-duplicate-calendar-1")) {
			// Ho attivato la duplicazione, la disattivo
			$(this).children("i").removeClass('icon-duplicate-calendar-1').addClass('icon-duplicate-calendar-0');

			$('.duplicate-calendar').children("i").removeClass('icon-check-0').removeClass('icon-check-1').addClass('icon-duplicate-calendar-0');

			// Nascondo il bottone di salvataggio
			$('.copy-calendar').attr('k_day', '').addClass('hidden');
		} else {
			// Attivo la copia
			$('.copy-calendar').attr('k_day', $(this).attr('k_day'));
			$('.duplicate-calendar').children("i").removeClass('icon-duplicate-calendar-0').addClass('icon-check-0');
			$(this).children("i").removeClass('icon-check-0').addClass('icon-duplicate-calendar-1');
		}
	}

	function setCalendar(programmazione) {
		app.utils.ajaxRequest("POST", app.utils.GetRisorsa("calendar"), {
			callback : function(data) {
				$('.copy-calendar').attr('k_day', '').addClass('hidden');

				if (data.success == true) {
					app.utils.setLocalConfig("programmazione", programmazione);
					
					app.f7.addNotification({
						title : Template7.global.lbl.DUPLICATE_SUCCESSFUL,
						hold : 2000
					});
					loadCalendar();
				} else {
					app.f7.addNotification({
						title : Template7.global.lbl.DUPLICATE_ERROR,
						hold : 2000
					});
				}
			},
			data : {
				calendar : programmazione
			}
		});
	}

	function copyCalendar() {
		var giorno = $(this).attr("k_day");

		// Aggiorno la programmazione
		$('.icon-check-1').each(function(i, obj) {
			programmazione.scheduling[$(this).parent().attr("k_day")].valori = programmazione.scheduling[giorno].valori;
		});

		// Salvo la nuova programmazione
		var new_programmazione = "";
		programmazione.scheduling.forEach(function(entry) {
			new_programmazione += entry.valori.join("");
		});
		setCalendar(new_programmazione);
	}

	return {
		init : init,
		refreshCalendar : refreshCalendar,
		splitCalendarPerDay : splitCalendarPerDay
	};
});