define([ "app", "text!js/devices/prerender.html", "text!js/devices/popup-devices-item.html", "text!js/devices/popup-devices.html", "text!js/devices/popup-labels.html", "text!js/devices/popup-label-devices.html", "text!js/devices/popup-label-settings.html", "js/devices/devicesView" ], function(app, Template, ItemTemplate, PopupTemplate, PopupLabelsTemplate, PopupLabelDevicesTemplate, PopupLabelsSettingsTemplate, View) {
	var bindings = [ {
		element : '.view-device',
		event : 'click',
		handler : viewDevice
	}, {
		element : '.edit-device',
		event : 'click',
		handler : editDevice
	}, {
		element : '.preferred-device',
		event : 'click',
		handler : preferredDevice
	}, {
		element : '.delete-device',
		event : 'click',
		handler : deleteDevice
	}, {
		element : '.popover-labels-device',
		event : 'click',
		handler : popoverLabelsDevice
	}, {
		element : '.view-label',
		event : 'click',
		handler : viewLabel
	}, {
		element : '.add-device',
		event : 'click',
		handler : addDevice
	} ];

	var devices = {}, ip_address = {}, interrupt = false, back = false, menu = false, labels = [], label_uid = -1;
	function init(parameters) {
		app.utils.Log("INIT devicesController");

		if (parameters.back != undefined) {
			back = parameters.back;
		} else {
			if (parameters.menu != undefined) {
				menu = parameters.menu;
			}
		}
		
		interrupt = false;

		// $('.screensaver').addClass("hidden");

		var compiled_template = Template7.compile(Template);
		var html = compiled_template({
			model : {
				back : back,
				menu : menu
			}
		});

		app.mainView.router.loadPage({
			content : html
		});

		loadDevices();

		// Abilito il pull-to-refresh per i miei dispositivi
		$('.pull-to-refresh-content').unbind('refresh');
		$('.pull-to-refresh-content').bind('refresh', function(e) {
			e.stopPropagation();
			refreshDevices();
		});

		app.utils.googleAnalytics('send', 'pageview', {
			'page' : 'devices'
		});

		app.f7.hideIndicator();
	}

	function reloadDevices() {
		setTimeout(function() {
			loadDevices();
		}, 500);
	}

	function loadDevices(parameters) {
		if (parameters == undefined) {
			parameters = {};
		}
		if (parameters.avoidRedirect == undefined) {
			parameters.avoidRedirect = false;
		}

		devices = app.utils.getDevices();
		if (devices == false) {
			devices = {};
		}
		
		labels = app.utils.getLabels();

		var current_device = app.utils.getCurrentDevice();
		for ( var key in devices) {
			if (key == current_device.UID) {
				devices[key].current = true;
			} else {
				devices[key].current = false;
			}
		}

		showDevices(parameters);
	}

	function refreshDevices(parameters) {
		if (parameters == undefined) {
			parameters = {};
		}
		if (parameters.avoidRedirect == undefined) {
			parameters.avoidRedirect = false;
		}
		if (parameters.uid == undefined) {
			parameters.uid = label_uid;
		}		
		
		app.f7.showIndicator();
		loadDevices(parameters);
		app.f7.pullToRefreshDone();
	}
	
	function getChildrenLabels(uid) {
		if(uid == undefined) {
			uid = -1;
		}
		
		var temp_labels = [];
		labels.forEach(function(lbl) {
			if(lbl.parent_uid == uid) {
				temp_labels.push(lbl);
			}
		});
		return temp_labels;
	}

	function getParentLabel() {
		if(label_uid == undefined) {
			label_uid = -1;
		}
		
		// Label
		var lbl = app.utils.getLabelFromUid(labels, label_uid);
		if(lbl === false) {
			return;
		}
		// Label padre
		var parent_lbl = app.utils.getLabelFromUid(labels, lbl.parent_uid);
		if(parent_lbl === false) {
			var uid = -1;
		} else {
			var uid = parent_lbl.uid;
		}
		
		updateBreadcrumbs(app.utils.getLabelFromUid(labels, uid));
		
		showDevices({
			uid : uid
		});
	}
	
	function showDevices(parameters) {
		if (parameters == undefined) {
			parameters = {};
		}
		if (parameters.avoidRedirect == undefined) {
			parameters.avoidRedirect = false;
		}
		if (parameters.uid == undefined) {
			parameters.uid = -1;
		}
		
		label_uid = parameters.uid;
		
		var devices2show = {};
		if(label_uid == -1) {
			// Rendo aggiungibili devices
			$('.add-device').addClass("open-popover");
			
			// Alla RADICE visualizzo tutto
			devices2show = devices; 
		} else {
			// Faccio sì che si debbano scegliere i device associabili alla label
			$('.add-device').removeClass("open-popover");
			
			// Visualizzo solo i dispositivi con label associata
			Object.keys(devices).forEach(function (key) {
				if(devices[key].hasLabel(label_uid)) {
					devices2show[key] = devices[key];
				}
			});
		}

		View.render({
			bindings : bindings,
			model : {
				devices : devices2show,
				labels : getChildrenLabels(parameters.uid)
			}
		});

		$('.back-hierarchy').unbind('click', getParentLabel);
		$('.back-hierarchy').bind('click', getParentLabel);
		
		if(label_uid == -1) {
			$('.back-hierarchy').hide();
		} else {
			$('.back-hierarchy').show();
		}
		
		// In caso di nessun prodotto avvio automaticamente la ricerca
		if (Object.keys(devices).length <= 0 && !parameters.avoidRedirect) {
			setTimeout(function() {
				app.f7.hideIndicator();
				scanNetwork();
			}, 500);
		} else {
			app.f7.hideIndicator();
		}
	}

	function scanNetworkRecursive(requestTemplate, current_n, final_n, parallelAmount) {
		if (!interrupt) {
			if (app.utils.checkConnection()) {

				var deferreds = [];
				for (i = 0; i < parallelAmount; i++) {
					url = requestTemplate.replace("{{progressCounter}}", parseInt(current_n + i));
					deferreds.push($.ajax(app.utils.ajaxRequest("GET", app.utils.GetRisorsa("", {
						url : url
					}), {
						callback : function(data) {
							if (data.success == true && data.UID != undefined) {
								var title = "";
								if (data.setup != undefined && data.setup.name != undefined && data.setup.serial != undefined) {
									if (data.setup.name == "") {
										title = Template7.global.lbl.NOT_CONFIGURED;
									} else {
										title = data.setup.name;
									}
								} else {
									title = Template7.global.lbl.NOT_CONFIGURED;
								}

								var compiled_template = Template7.compile(ItemTemplate);
								var html = compiled_template({
									model : {
										name : title,
										UID : data.UID,
										serial : data.setup.serial,
										url : this.url,
										standalone : data.RESULT.ns
									}
								});
								$('.devices-found-container ul').append(html);
							}
						},
						timeout : 10000,
						waitRequest : true,
						extraData : "network",
						returnObject : true,
						skipNotification : true
					})));
				}

				jQuery.whenAll.apply(null, deferreds).then(function(results) {
					/* Tutte a buon fine */
					if (current_n + parallelAmount <= final_n - 2) {
						scanNetworkRecursive(requestTemplate, current_n + parallelAmount, final_n, parallelAmount);
					} else {
						app.f7.hidePreloader();
					}
				}, function(results) {
					/* Tutte le chiamate effettuate, una o più sono fallite */
					if (current_n + parallelAmount <= final_n - 2) {
						var progress_percent = ((current_n + parallelAmount) - Template7.global.scanRangeStart) / (final_n - Template7.global.scanRangeStart) * 100;
						app.$$('.modal-preloader .modal-title').html(Template7.global.lbl.NETWORK_SCAN + " (" + (current_n + parallelAmount) + "/" + final_n + ")");
						app.f7.setProgressbar(app.$$('.modal-preloader'), progress_percent);

						scanNetworkRecursive(requestTemplate, current_n + parallelAmount, final_n, parallelAmount);
					} else {
						interrupt = true;

						if ($('.devices-found-container ul li').size() <= 0) {
							$('.devices-found-container').html('').append("<div class='content-block'><p>" + Template7.global.lbl.NO_DEVICES + ".</p></div>");
						}

						app.f7.hidePreloader();
						app.f7.hideProgressbar();
						app.$$('.modal-preloader .modal-title').html(Template7.global.lbl.NETWORK_SCAN + " (" + final_n + "/" + final_n + ")");

						bindFoundDevices();
					}
				});
			} else {
				app.f7.hidePreloader();
				app.f7.hideProgressbar();
				
				// Termina tutte le richieste ajax pendenti
				$.ajaxQ.abortAll();
				
				app.utils.noConnectionModal();
				interrupt = true;
			}
		}
	}

	function bindFoundDevices() {
		$('.devices-found-container .select-device').unbind('click');
		$('.devices-found-container .select-device').bind('click', selectDevice);

		$('.page[data-page="popup-devices"] .scan-network').unbind('click');
		$('.page[data-page="popup-devices"] .scan-network').bind('click', function() {
			$('.devices-found-container').html('<div class="list-block"><ul></ul></div>');
			scanNetworkDo();
		});

		$('.page[data-page="popup-devices"] .cerca-ancora').unbind('click');
		$('.page[data-page="popup-devices"] .cerca-ancora').bind('click', function() {
			app.utils.networkConnection({
				app : app
			});
		});
		
		// Reload dell'app in caso di indietro da scansione della rete
		$('.page[data-page="popup-devices"] .scan-back').unbind('click');
		$('.page[data-page="popup-devices"] .scan-back').bind('click', function() {
			window.location.reload();
		});		
	}

	function scanNetworkDo() {
		// Svuoto la lista dei dispositivi trovati
		$('.devices-found-container ul li').html('');

		interrupt = false;
		app.f7.showPreloader(Template7.global.lbl.NETWORK_SCAN + " (" + Template7.global.scanRangeStart + "/" + Template7.global.scanRangeStop + ")");

		// Progress bar
		var container = app.$$('.modal-preloader');
		if (container.children('.progressbar, .progressbar-infinite').length) {
		} else {
			app.f7.showProgressbar(container, 0, 'blue');
		}

		// Bottone terminare
		app.$$('.modal-preloader .modal-text').append('<a href="#" class="button stop-scan button-fill button-raised">' + Template7.global.lbl.STOP + '</a>');

		$('.stop-scan').unbind('click');
		$('.stop-scan').bind('click', function() {
			interrupt = true;

			app.f7.hidePreloader();

			// Termina tutte le richieste ajax pendenti
			$.ajaxQ.abortAll();

			if ($('.devices-found-container ul li').size() <= 0) {
				$('.devices-found-container').html('').append("<div class='content-block'><p>" + Template7.global.lbl.NO_DEVICES + ".</p><div class='row'><div class='col-100 tablet-33'></div><div class='col-100 tablet-33'>" + "<a class='button button-big button-raised button-inverse button-custom close-popup cerca-ancora' href='#'>" + Template7.global.lbl.ADD_MANUALLY+ "</a></div><div class='col-100 tablet-33'></div></div></div>");
			}

			bindFoundDevices();
		});

		var ip_address = $('.ip-address').html();
		var str_ip = ip_address.split(".");
		var requestTemplate = "http://" + str_ip[0] + '.' + str_ip[1] + '.' + str_ip[2] + ".{{progressCounter}}/api/v/1/status";

		// Possibilità di eseguire N=4 richieste Ajax parallelizzate
		scanNetworkRecursive(requestTemplate, Template7.global.scanRangeStart, Template7.global.scanRangeStop, Template7.global.scanThreads);
	}

	function scanNetworkAfterIP(ip_address) {
		var compiled_template = Template7.compile(PopupTemplate);
		var html = '<div class="popup popup-modes">' + compiled_template({
			model : {
				ip_address : ip_address
			}
		}) + '</div>';

		app.f7.popup(html, true);

		setTimeout(function() {
			scanNetworkDo();
		}, 500);

		app.f7.hideIndicator();
	}

	function scanNetwork() {
		app.f7.closeModal();
		
		if (app.utils.checkConnection() && app.utils.checkWifi()) {
			app.f7.showIndicator();
			setTimeout(function() {
				app.utils.findIP(scanNetworkAfterIP);
			}, 500);
		} else {
			app.utils.noConnectionModal();
		}
	}

	function doEditDevice(uid, serial, name) {
		app.f7.modal({
			title : Template7.global.lbl.EDIT_DEVICE,
			afterText : '<div class="input-field"><input type="text" id="device_name" pattern="[a-zA-Z0-9]" class="modal-text-input" value="' + name + '" required></div>',
			buttons : [ {
				text : Template7.global.lbl.CANCEL
			}, {
				text : Template7.global.lbl.MODAL_BUTTON_OK,
				bold : true,
				onClick : function() {
					var new_name = $('#device_name').val().trim(), re = new RegExp("^([a-zA-Z0-9-_. ]{1,})$");

					// Controllo che il nome del dispositivo sia fatto solo da
					// lettere e numeri
					if (re.test(new_name)) {
						var url = app.utils.GetRisorsa("setup"),
							cur_dev = app.utils.getCurrentDevice();
						if(cur_dev.getStandalone() == false) {
							url = app.utils.GetRisorsa("setup_advanced");
						}
						app.utils.ajaxRequest("POST", url, {
							callback : function(data) {
								if (data.success == true) {
									devices[uid].name = new_name;

									app.utils.saveDevices(devices);

									// Refresh della vista
									loadDevices();
								} else {
									app.f7.addNotification({
										title : Template7.global.lbl.ERROR + ": " + data.error,
										hold : 2000
									});
								}
							},
							data : {
								serial : serial,
								name : new_name
							}
						});
					} else {
						app.f7.addNotification({
							title : Template7.global.lbl.ERROR + ": " + Template7.global.lbl.ONLY_ALPHABETIC_AND_DIGITS,
							hold : 3000
						});
					}
				}
			} ]
		});
		app.utils.focusField('device_name');
	}
	
	function editDevice() {
		var curr_dev = app.utils.getCurrentDevice(),
			uid = $(this).attr('k_uid'), serial = devices[uid].serial, name = devices[uid].name;
		if(curr_dev.getStandalone() == false) {			
			app.utils.ajaxRequest("GET", app.utils.GetRisorsa("status_advanced"), {
				callback : function(data) {
					if (data.success == true && data.net != undefined) {
						doEditDevice(uid, serial, name);
					} else {
						app.f7.addNotification({
							title : Template7.global.lbl.ERROR + ": " + data.error,
							hold : 2000
						});
					}
				}, callback_error : function(jqXHR, textStatus, errorThrown) {
					app.f7.addNotification({
						title : Template7.global.lbl.ERROR + ": " + jqXHR.statusText,
						hold : 2000
					});
				},
				timeout : 10000,
				skipNotification : true,
				extraData : "cloud"
			});
		} else {
			doEditDevice(uid, serial, name);
		}
	}

	function viewDevice() {
		var uid = $(this).attr('k_uid');

		var device = app.utils.getDeviceByUID(uid);
		app.utils.setCurrentDevice(device);

		app.utils.checkSetup(app);
	}
	
	function viewLabel() {
		var uid = $(this).attr('k_uid');
		
		updateBreadcrumbs(app.utils.getLabelFromUid(labels, uid));
		
		refreshDevices({
			uid : uid
		});
	}
	
	function updateBreadcrumbs(label) {
		var title = label.extended_title;
		if(label === false) {
			title = Template7.global.lbl.MY_PRODUCTS;
		}
		$('.header-title').html(title);
	}

	function preferredDevice() {
		var UID = $(this).attr('k_uid'), name = $(this).attr('k_name');

		app.utils.setPreferredDevice(UID);

		app.f7.addNotification({
			title : Template7.global.lbl.DEVICE + ": " + name + " " + Template7.global.lbl.SET_AS_PREFERRED,
			hold : 2000
		});

		refreshDevices();
	}

	function selectDevice() {
		if ($(this).hasClass('icon-add-device')) {
			// In caso non avessi devices già configurati lo imposto come preferito di default
			var devices = app.utils.getDevices(), 
				preferred = true;
			if (devices !== false && Object.keys(devices).length > 0) {
				preferred = false;
			}
			
			var dev = {
				"UID" : $(this).attr('k_uid'),
				"preferred" : preferred,
				"name" : $(this).attr('k_name'),
				"serial" : $(this).attr('k_serial'),
				"standalone" : $(this).attr('k_standalone') == "1" ? true : false
			};

			// Se ho il dispositivo mantengo il suo stato di preferito in caso ci sia
			var old_dev = app.utils.getDeviceByUID($(this).attr('k_uid'));
			if (old_dev != false) {
				dev.preferred = old_dev.preferred;
			}

			var dev2_save = app.utils.getDeviceModel(dev);
			dev2_save.save(app);

			// Se l'unico lo imposto come corrente, altrimenti non faccio nulla
			if(preferred) {
				app.utils.setCurrentDevice(dev2_save);
			}
			
			// Lo seleziono
			$(this).removeClass('icon-add-device').addClass('icon-check-1');
			
			// moreno.trainini, 09-06-2017
			// Non faccio più il setup	
			/*app.f7.addNotification({
				title : Template7.global.lbl.DEVICE+" '"+$(this).attr('k_name')+"' "+Template7.global.lbl.ADDED_SUCCESSFULLY,
				hold : 2000
			});		*/	
			//app.utils.checkSetup(app);
		}
	}

	function deleteDevice() {
		var uid = $(this).attr('k_uid');

		app.f7.confirm(Template7.global.lbl.SURE_TO_DELETE_DEVICE, function() {
			app.utils.deleteDevice(uid, app);
			// Ricarico la lista di dispositivi
			reloadDevices();
		});
	}
	
	function saveLabels() {
		var uid = $('input[name="uid"]').val();
		
		devices[uid].emptyLabels();
		$('input[name="labels"]:checked').each(function() {
			devices[uid].addLabel(this.value);
		});
		app.utils.saveDevices(devices);
	}
	
	function saveDevices() {
		var lbl_uid = $('input[name="uid"]').val(),
			devices_uid = [];
		$('input[name="devices"]:checked').each(function() {
			devices_uid.push(this.value);
		});
		
		Object.keys(devices).forEach(function (key) {
			if(app.utils.inArray(key, devices_uid)) {
				devices[key].addLabel(lbl_uid);
			} else {
				devices[key].removeLabel(lbl_uid);
			}			
		});
		
		app.utils.saveDevices(devices);
		
		showDevices({
			uid : lbl_uid
		});
	}
	
	function popoverLabelsDevice() {
		var uid = $(this).attr('k_uid');
		var clickedLink = this;
		var popoverHTML = '<div class="popover popover-help">'+
	  					  '<div class="popover-angle"></div>'+
	                      '<div class="popover-inner">'+
	                        '<div class="list-block">'+
			                	'<ul>'+
			                    	'<li><a href="#" k_uid="'+uid+'" class="list-button close-popover item-link assign-labels">'+Template7.global.lbl.ASSIGN_LABELS+'</a></li>'+
			                        '<li><a href="#" k_uid="'+uid+'" class="list-button close-popover close-popover item-link copy-settings">'+Template7.global.lbl.COPY_SETTINGS+'</a></li>'+
			                    '</ul>'+
			                  '</div>'+
	                      '</div>'+
	                    '</div>';
	    app.f7.popover(popoverHTML, clickedLink);
	    
	    $('.assign-labels').unbind('click', labelsDevice);
	    $('.assign-labels').bind('click', labelsDevice);
	    
	    $('.copy-settings').unbind('click', copySettingsDevice);
	    $('.copy-settings').bind('click', copySettingsDevice);
	}
	
	function copySettingsDevice() {
		var uid = $(this).attr('k_uid'),
			dev = app.utils.getDeviceByUID(uid);
				
		var compiled_template = Template7.compile(PopupLabelsSettingsTemplate);
		var html = compiled_template({
			device : dev,
			labels : labels
		});
		
		app.f7.popup(html, true);
		
		$('.save-settings').unbind('click', saveSettingsLabels);
		$('.save-settings').bind('click', saveSettingsLabels);
	}
	
	function saveSettingsLabels() {
		var uid = $('input[name="uid"]').val(),
			dev = app.utils.getDeviceByUID(uid);
		
		app.f7.showIndicator();
		
		// Imposto tutto a caricamento
		$('input[name="labels"]:checked').parent().children(".item-media").html('<i class="icon icon-hourglass"></i>');
		$('input[name="labels"]:not(:checked)').parent().children(".item-media").html('');
		
		var copyProgress={};
		
		// Ottengo lo stato del dispositivo sorgente
		dev.getStatus({
			callback_success : function(data) {				
				if(data.RESULT != undefined) {
					// Ottengo la programmazione del dispositivo sorgente
					dev.getProgram({
						callback_success : function(data_program) {
							app.utils.Log(data_program);
							
							if(data_program.success == true && data_program.RESULT != undefined) {
								var data_final = data.RESULT;
								data_final.program = data_program.RESULT;
								
								// Nascondo il bottone di salvataggio
								$('.save-settings').hide();
								
								// Per ogni label che ho selezionato
								$('input[name="labels"]:checked').each(function() {
									var lbl_uid = this.value;
									
									// Imposto la visualizzazione della clessidra
									$(this).parent().children(".item-media").html('<i class="icon icon-loading"></i>');
									
									var that=this;
									
									if(copyProgress[lbl_uid]===undefined){
										copyProgress[lbl_uid]={
											error:false,
											devices2Copy:0,
											devicesCopied:0,
											errorCounter:0,
											successCounter:0,
											requestsCounter:{}
										};
									}
									
									Object.keys(devices).forEach(function (key) {
										var d = app.utils.getDeviceByUID(key);
										
										// moreno.trainini, 07-06-2017
										// Aggiunto controllo per non copiare le impostazioni sul dispositivo stesso (se associato a più label)
										if(d.hasLabel(lbl_uid) && d.getUID() != data.UID) {		
											copyProgress[lbl_uid].devices2Copy++;
											copyProgress[lbl_uid].requestsCounter[key]=0;
											
											app.utils.Log("copio le impostazioni sul device "+key+" nella label "+lbl_uid);
											
											dev.copy(data_final, d, {
												callback_success : function(data) {
													app.utils.Log(copyProgress[lbl_uid]);

													copyProgress[lbl_uid].successCounter++;
													
													if(!copyProgress[lbl_uid].error){
														app.utils.Log("ok, ho copiato con successo "+key+" nella label "+lbl_uid);
														copyProgress[lbl_uid].requestsCounter[key]++;
														
														if(copyProgress[lbl_uid].requestsCounter[key]==6){
															app.utils.Log("ho copiato completamente il device "+key);
															copyProgress[lbl_uid].devicesCopied++;
														}
														
														if(copyProgress[lbl_uid].devicesCopied==copyProgress[lbl_uid].devices2Copy){
															app.utils.Log("ho copiato completamente la label "+lbl_uid);
															$(that).parent().children(".item-media").html('<i class="icon icon-success"></i>');
														}														
													}else{
														app.utils.Log("ci sono stati degli errori nella copia per la label "+lbl_uid);														
													}													
												},
												callback_error : function(error) {
													app.utils.Log("ko, c'è stato un errore nella copia da "+key+" nella label "+lbl_uid);													

													copyProgress[lbl_uid].errorCounter++;
													copyProgress[lbl_uid].error=true;
													
													$(that).parent().children(".item-media").html('<i class="icon icon-error"></i>');
												},
												utils : app.utils
											});											
										} else {
											app.utils.Log("ok, non ho sovrascritto i valori per lo stesso device");
											copyProgress[lbl_uid].requestsCounter[key]++;
											
											if(copyProgress[lbl_uid].requestsCounter[key]==6){
												copyProgress[lbl_uid].devicesCopied++;
											}
											
											if(copyProgress[lbl_uid].devicesCopied==copyProgress[lbl_uid].devices2Copy){
												$(that).parent().children(".item-media").html('<i class="icon icon-success"></i>');
											}
										}
									});
								});
							}
							app.f7.hideIndicator();
						},
						callback_error : function(error) {
							// Imposto tutto ad errore perché non riesco a leggere la programmazione del dispositivo sorgente
							$('input[name="labels"]:checked').parent().children(".item-media").html('<i class="icon icon-error"></i>');
						},
						utils : app.utils
					});				
				}
			},
			callback_error : function(error) {
				// Imposto tutto ad errore perché non riesco a leggere lo stato del dispositivo sorgente
				$('input[name="labels"]:checked').parent().children(".item-media").html('<i class="icon icon-error"></i>');
			},
			utils : app.utils
		});
	}
	
	function labelsDevice() {
		var uid = $(this).attr('k_uid'),
			dev = app.utils.getDeviceByUID(uid);
		
		var device_labels = [];
		labels.forEach(function(lbl) {
			// Checked
			lbl.checked = dev.hasLabel(lbl.uid);
			device_labels.push(lbl);
		});
		
		var compiled_template = Template7.compile(PopupLabelsTemplate);
		var html = compiled_template({
			device : dev,
			labels : device_labels
		});
		
		app.f7.popup(html, true);
		
		$('.save-labels').unbind('click', saveLabels);
		$('.save-labels').bind('click', saveLabels);
	}
	
	function devicesLabel() {		
		var label_devices = [];
		Object.keys(devices).forEach(function (key) {			
			var d = devices[key];
			// Checked
			d.checked = d.hasLabel(label_uid);
			label_devices.push(d);
		});
		
		var lbl = app.utils.getLabelFromUid(labels, label_uid);
		
		var compiled_template = Template7.compile(PopupLabelDevicesTemplate);
		var html = compiled_template({
			label : lbl,
			devices : label_devices
		});
		
		app.f7.popup(html, true);
		
		$('.save-devices').unbind('click', saveDevices);
		$('.save-devices').bind('click', saveDevices);
	}
	
	function addDevice() {
		if($(this).hasClass("open-popover")) {
			// Non faccio nulla
		} else {
			// Visualizzo l'elenco di devices associabili
			devicesLabel();
		}
	}

	return {
		init : init,
		scanNetwork : scanNetwork
	};
});