define([ "app", "text!js/labels/prerender.html", "js/labels/labelsView", "text!js/labels/popup-label.html" ], function(app, Template, View, LabelTemplate) {
	var bindings = [{
		element : '.add-label',
		event : 'click',
		handler : showPopupAddLabel
	}, {
		element : '.delete-label',
		event : 'click',
		handler : deleteLabel
	}, {
		element : '.edit-label',
		event : 'click',
		handler : showPopupEditLabel
	}];
	var labels = [];
	function init(parameters) {
		app.utils.Log("INIT labelsController");

		var compiled_template = Template7.compile(Template);
		var html = '<div class="popup">' + compiled_template({}) + '</div>';

		app.mainView.router.loadPage({
			content : html
		});
		
		loadLabels();

		app.utils.googleAnalytics('send', 'pageview', {
			'page' : 'labels'
		});
		
		// Abilito il pull-to-refresh per i miei dispositivi
		$('.pull-to-refresh-content').unbind('refresh');
		$('.pull-to-refresh-content').bind('refresh', function(e) {
			e.stopPropagation();
			refreshLabels();
		});

		app.f7.hideIndicator();
	}

	function loadLabels() {
		getLabels();		
	}	
	
	function showLabels(data) {		
		View.render({
			bindings : bindings,
			model : {
				labels : labels
			}
		});		
		app.f7.hideIndicator();
	}
	
	function getLabels() {
		labels = app.utils.getLabels();
		showLabels(labels);
	}
	
	function getLabelFromUid(uid) {
		return app.utils.getLabelFromUid(labels, uid);
	}
	
	function setLabel(label) {
		var labels_temp = [];
		labels.forEach(function(l) {
			if(l.uid == label.uid) {
				l = label;
			}
			labels_temp.push(l);
		});
		labels = labels_temp;
	}
	
	function labelPruneTree(label, label2remove) {
		if(label.uid == label2remove.uid) {
			return false;
		}
		if(label.parent_uid == label2remove.uid) {
			label.parent_uid = label2remove.parent_uid;
		}
		return label;
	}
	
	function isParentRec(label, uid) {
		if(label.uid == uid) {
			return true;
		}
		if(label.parent_uid != -1) {
			return isParentRec(getLabelFromUid(label.parent_uid), uid);
		}		
	}
	
	function deleteLabelFromUid(uid, parameters) {
		if(parameters == undefined) {
			parameters = {};
		}
		
		var label2remove = getLabelFromUid(uid), 
			temp_labels = [];
		
		labels.forEach(function(l) {
			var lbl = labelPruneTree(l, label2remove);
			if(lbl !== false) {
				temp_labels.push(lbl);
			}			
		});
		
		temp_labels = app.utils.getHierarchyLabels(temp_labels);
		
		if(parameters.RETURN != undefined && parameters.RETURN) {
			return temp_labels;
		}
		
		labels = temp_labels;
		
		saveLabels();
	}
	
	function saveLabel() {
		var uid =$('input[name="uid"]').val(),
			title = $('input[name="title"]').val(),
			parent_uid = $('select[name="parent_uid"]').val();
		
		var label = app.utils.getLabelModel({
			uid : uid,
			title : title,
			parent_uid : parent_uid
		});
		
		if(label.uid == -1) {
			// Nuova label
			label.uid = getLabelUid();			
			labels.push(label);
		} else {
			// Modifica label
			setLabel(label);
		}
		
		saveLabels();
	}
	
	function saveLabels() {
		// write the JSON file
		app.utils.saveLabels(labels);
		
		refreshLabels();
	}
	
	function getLabelUid() {
	     return Math.random().toString(36).substr(2, 10);
	}

	function getParents(uid) {
		var parents = [];
		labels.forEach(function(l) {
			if(!isParentRec(l,uid)) {
				parents.push(l);
			}		
		});
		
		parents.unshift(app.utils.getLabelModel({
			title : Template7.global.lbl.LABEL_ROOT
		}));
		return parents;
	}
	
	function popupLabel(label) {
		var parents = getParents(label.uid);
		var compiled_template = Template7.compile(LabelTemplate);
		var popupHTML = compiled_template({
			model : {
				parents : parents,
				label : label,
				title : (label.uid == -1) ? Template7.global.lbl.LABEL_NEW : Template7.global.lbl.LABEL_EDIT
			}
		});		
        app.f7.popup(popupHTML, true, true);
        
        $('.save-label').unbind('click', saveLabel);
        $('.save-label').bind('click', saveLabel);
        
        setTimeout(function() {
        	$('input[name="title"]').focus();
        }, 500);        
	}
	
	function showPopupAddLabel() {
		var label = app.utils.getLabelModel({});
		label.parent_title = Template7.global.lbl.LABEL_ROOT;	
		popupLabel(label);
	}
	
	function showPopupEditLabel() {
		var uid = $(this).attr('k_uid');
		var label = getLabelFromUid(uid);
		
		if(label != false) {
			if(label.parent_uid == -1) {
				var parent = app.utils.getLabelModel({
					title : Template7.global.lbl.LABEL_ROOT
				});
			} else {
				var parent = getLabelFromUid(label.parent_uid);
			}			
			label.parent_title = parent.title;			
			popupLabel(label);
		}		
	}
	
	function deleteLabelFromDevices(uid) {
		var devices = app.utils.getDevices();
		
		var temp_devices = {};
		Object.keys(devices).forEach(function (key) {
			if(devices[key].hasLabel(uid)) {
				devices[key].removeLabel(uid);
			}
			temp_devices[key] = devices[key];
		});
		
		app.utils.saveDevices(temp_devices);
	}
	
	function deleteLabel() {
		var uid = $(this).attr('k_uid');
		app.f7.confirm(Template7.global.lbl.SURE_TO_DELETE_LABEL, function() {
			deleteLabelFromUid(uid);
			
			deleteLabelFromDevices(uid);
		}, function() {
		});
	}
	
	function refreshLabels(parameters) {
		if (parameters == undefined) {
			parameters = {};
		}
		if (parameters.avoidRedirect == undefined) {
			parameters.avoidRedirect = false;
		}
		app.f7.showIndicator();
		loadLabels();
		app.f7.pullToRefreshDone();
	}

	return {
		init : init
	};
});