define([ 'app' ], function(app) {

	var app = null;
	
	function Device() {}

	// Setter
	Device.prototype.setUID = function(uid) {
		this.UID = uid || '';
	};

	Device.prototype.setSerial = function(serial) {
		this.serial = serial || '';
	};

	Device.prototype.setName = function(name) {
		this.name = name || '';
	};
	
	Device.prototype.setPassword = function(password) {
		this.password = password || '';
	};

	Device.prototype.setPreferred = function(preferred) {
		this.preferred = (preferred == undefined) ? false : preferred;
	};

	Device.prototype.setStandalone = function(standalone) {
		this.standalone = (standalone == undefined) ? true : standalone;
	};

	Device.prototype.setFirmwareVersion = function(firmware_version) {
		this.firmware_version = firmware_version;
	};

	Device.prototype.setUrl = function(url) {
		this.url = url;
	};
	
	Device.prototype.setIP = function(ip_address) {
		this.ip = ip_address;
	}

	// Getter
	Device.prototype.getUID = function() {
		return this.UID;
	};

	Device.prototype.getSerial = function() {
		return this.serial;
	};

	Device.prototype.getName = function() {
		return this.name;
	};
	
	Device.prototype.getPassword = function() {
		return this.password;
	};

	Device.prototype.getPreferred = function() {
		return this.preferred;
	};

	Device.prototype.getStandalone = function() {
		return this.standalone;
	};

	Device.prototype.getFirmwareVersion = function() {
		return this.firmware_version;
	};

	Device.prototype.getUrl = function() {
		if (this.standalone == true) {
			this.url = Template7.global.defaultUrl;
		} else {
			this.url = Template7.global.defaultUrlCloud;
		}

		if (Template7.global.localTest == true || Template7.global.testVersion == true) {
			this.url = Template7.global.defaultUrl;
		}
		
		return this.url;
	};
	
	Device.prototype.getIP = function() {
		return this.ip;
	}

	// Aggiunta device al localStorage
	Device.prototype.save = function(app) {
		var devices = app.utils.getDevices();
		if (devices == false) {
			app.utils.Log("No devices present, inserting the first one as preferred");
			this.setPreferred(true);
		}
		app.utils.saveDevice(this);
	};

	// Caricamento del device
	Device.prototype.load = function(values) {
		this.name = values['name'] || '';
		this.serial = values['serial'] || '';
		this.UID = values['UID'] || '';
		this.preferred = (values['preferred'] == undefined) ? false : values['preferred'];
		this.standalone = (values['standalone'] == undefined) ? true : values['standalone'];
		this.password = values['password'] || '';
		this.ip = values['ip'] || '';

		// versione
		this.firmware_version = values['firmware_version'] || '1.0';
		
		// labels
		this.labels = values['labels'] || [];
	};
	
	// Controllo associazione label-device
	Device.prototype.hasLabel = function(uid) {
		for (i = 0; i < this.labels.length; i++) {
			if (this.labels[i] == uid) {
				return true;
			}
		}
		return false;
	};
	
	Device.prototype.addLabel = function(label) {
		this.removeLabel(label);
		
		this.labels.push(label);
	};
	
	Device.prototype.removeLabel = function(uid) {
		var index = this.labels.indexOf(uid);
		if (index > -1) {
		    this.labels.splice(index, 1);
		}
	};
	
	Device.prototype.emptyLabels = function() {
		this.labels = [];
	};
	
	Device.prototype.getStatus = function(parameters) {
		if(parameters === undefined) {
			parameters = {};
		}
		
		var params2call = {};
		params2call.utils = parameters.utils;
		params2call.device = this;
		
		if(parameters.callback_success !== undefined) {
			params2call.callback_success = parameters.callback_success;
		}
		if(parameters.callback_error !== undefined) {
			params2call.callback_error = parameters.callback_error;
		}
		
		parameters.utils.getStatus(params2call);
	};
	
	Device.prototype.getProgram = function(parameters) {
		if(parameters === undefined) {
			parameters = {};
		}
		
		var params2call = {};
		params2call.utils = parameters.utils;
		params2call.device = this;
		
		if(parameters.callback_success !== undefined) {
			params2call.callback_success = parameters.callback_success;
		}
		if(parameters.callback_error !== undefined) {
			params2call.callback_error = parameters.callback_error;
		}
		
		parameters.utils.ajaxRequest("GET", parameters.utils.GetRisorsa("calendar", params2call), params2call);
	};
	
	Device.prototype.setMode = function(call, value, parameters) {
		if(parameters === undefined) {
			parameters = {};
		}
		
		var params2call = {};
		
		if(parameters.callback_success !== undefined) {
			params2call.callback_success = parameters.callback_success;
		}
		if(parameters.callback_error !== undefined) {
			params2call.callback_error = parameters.callback_error;
		}
		if(parameters.device !== undefined) {
			params2call.device = parameters.device;
		}
		
		return parameters.utils.setMode(call, value, "copy", params2call);
	};
	
	Device.prototype.copy = function(data, dev_to, parameters) {
		if(parameters === undefined) {
			parameters = {};
		}
		
		var params2call = {};
		params2call.utils = parameters.utils;
		params2call.device = dev_to;
		params2call.returnObject = true;
		
		if(parameters.callback_success !== undefined) {
			params2call.callback_success = parameters.callback_success;
		}
		if(parameters.callback_error !== undefined) {
			params2call.callback_error = parameters.callback_error;
		}
		
		var workingMode = parameters.utils.getWorkingMode(data.wm),
			params2save = [{
			call : 'set/setpoint',
			value : 'sp'
		}, {
			call : 'set/fan',
			value : 'fs'
		}, {
			call : 'set/feature/rotation',
			value : 'fr'
		}, {
			call : 'set/feature/night',
			value : 'nm'
		}, {
			call : 'set/calendar',
			value : 'cm'
		}, {
			call : 'set/mode/'+workingMode,
			value : 'wm'
		}, {
			call : 'calendar/program',
			value : 'program'
		}, {
			call : 'power',
			value : 'ps'
		}];
		
		// Coda di richieste ajax
		var ajaxManager = parameters.utils.ajaxManager();
		
		params2save.forEach(function(p) {
			if(data[p.value] !== undefined) {
				ajaxManager.addReq(dev_to.setMode(p.call, data[p.value], params2call));
			}
		});
		
		// Eseguo la coda di richieste ajax
		ajaxManager.run();
	};

	return Device;
});