define([ "app", "text!js/networks/prerender.html", "js/networks/networksView" ], function(app, Template, View) {
	var bindings = [ {
		element : '.set-network',
		event : 'click',
		handler : setNetwork
	} ];

	var standalone = false;

	function init(parameters) {
		app.utils.Log("INIT networksController");

		if (parameters != undefined && parameters.standalone != undefined) {
			standalone = parameters.standalone;
		}

		var compiled_template = Template7.compile(Template);
		var html = compiled_template({
			standalone : standalone
		});

		app.mainView.router.loadPage({
			content : html
		});

		app.utils.googleAnalytics('send', 'pageview', {
			'page' : 'networks'
		});

		$('.iniziamo').unbind('click');
		$('.iniziamo').bind('click', function() {
			loadNetworks();
		});
		app.f7.hideIndicator();
	}

	function removeDuplicatedNetworks(networks) {
		var temp_networks = [];
		networks.forEach(function(n) {
			if (!app.utils.inArray(n.essid, networks)) {
				temp_networks.push(n);
			}
		});
		return temp_networks;
	}
	
	// Escludo le reti che non passano la reg. expression
	function filterNetworks(networks) {
		var pattern = /^[a-z0-9.-_]+$/i;		
		for (var index in networks) {
			var disabled = false;
			if(!pattern.test(networks[index].essid)) {
				disabled = true;
			}
			networks[index].disabled = disabled;
		}
		return networks;
	}

	function loadNetworks() {
		app.utils.ajaxRequest("GET", app.utils.GetRisorsa("network_scan"), {
			callback : function(data) {
				if (data.success == true) {
					var networks = removeDuplicatedNetworks(data.RESULT);
					// Filtro le reti escludendo quelle con simboli non supportati
					networks = filterNetworks(networks);
					showNetworks(networks);
				} else {
					app.f7.addNotification({
						title : Template7.global.lbl.ERROR + ": " + data.error,
						hold : 2000
					});
				}
			}
		});
	}

	function refreshNetworks() {
		app.f7.showIndicator();
		loadNetworks();
		app.f7.pullToRefreshDone();
	}

	function sortNetworks(data) {
		data.sort(function(a, b) {
			return a.signal < b.signal;
		});
		return data;
	}

	function showNetworks(data) {
		// Ordino le reti da quella con più segnale a quella con meno segnale
		data = sortNetworks(data);

		View.render({
			bindings : bindings,
			model : data
		});

		// Abilito il pull-to-refresh per le wifi trovate
		app.$$('.networks-content').addClass('pull-to-refresh-content');
		$('.pull-to-refresh-content').unbind('refresh');
		$('.pull-to-refresh-content').bind('refresh', function(e) {
			e.stopPropagation();
			refreshNetworks();
		});
	}

	function sendNetworkConfigRecursive(connected, interrupt, i, max, timeout_s, essid) {
		app.f7.setProgressbar(app.$$('.modal-preloader'), (i / max) * 100);

		if (i < max && !interrupt) {
			$('.modal-preloader .modal-title').html(Template7.global.lbl.CONNECTION_TRY + " " + (i + 1) + "/" + max);
			// Richiesta AJAX Cloud per configurazione wifi
			var start = new Date().getTime();

			app.utils.ajaxRequest("GET", app.utils.GetRisorsa("connection", {
				url : Template7.global.defaultUrlCloud
			}), {
				callback : function(data) {
					if (data.success == true) {

						app.utils.ajaxRequest("GET", app.utils.GetRisorsa("waitfordevice", {
							url : Template7.global.defaultUrlCloud
						}), {
							callback : function(data2) {
								if (data2.success == true) {
									interrupt = true;
									var device = app.utils.getCurrentDevice();
									device.setStandalone(false);

									device.save(app);

									app.f7.hidePreloader();
									app.f7.hideProgressbar();

									app.utils.Log("Passaggio a connected: " + interrupt);

									$(connected).removeClass("hidden");
									if (interrupt) {
										$(connected).children("i").addClass("icon-network-set").removeClass("icon-network-fail");

										app.utils.Log("Dispositivo configurato con successo");

										// Visualizzo il nuovo device
										app.f7.showPreloader(Template7.global.lbl.CONFIGURATION_IN_PROGRESS);
										setTimeout(function() {
											// Consento al cellulare di andare
											// in sleep mode
											app.utils.allowSleepAgain();

											app.f7.hidePreloader();

											app.utils.Log("Carico la checkSetup");
											app.utils.checkSetup(app);
										}, 10000);
									} else {
										$(connected).children("i").addClass("icon-network-fail").removeClass("icon-network-set");
									}
								} else {
									setTimeout(function() {
										sendNetworkConfigRecursive(connected, false, i + 1, max, timeout_s, essid);
									}, timeout_s);
								}
							},
							callback_error : function(jqXHR, textStatus, errorThrown) {
								setTimeout(function() {
									sendNetworkConfigRecursive(connected, false, i + 1, max, timeout_s, essid);
								}, timeout_s);
							},
							waitRequest : true,
							extraData : "cloud",
							crossDomain : true,
							skipNotification : true,
							timeout : timeout_s
						});
					} else {
						setTimeout(function() {
							sendNetworkConfigRecursive(connected, false, i + 1, max, timeout_s, essid);
						}, timeout_s);
					}
				},
				callback_error : function(jqXHR, textStatus, errorThrown) {
					setTimeout(function() {
						sendNetworkConfigRecursive(connected, false, i + 1, max, timeout_s, essid);
					}, timeout_s);
				},
				waitRequest : true,
				skipNotification : true,
				timeout : timeout_s,
				crossDomain : true,
				extraData : "cloud"
			});
		} else {
			if (!interrupt) {
				app.f7.hidePreloader();
				app.f7.hideProgressbar();

				var dev = app.utils.getCurrentDevice(), rete = Template7.global.ESSID_default + "_" + dev.getName();
				$(connected).children("i").addClass("icon-network-fail").removeClass("icon-network-set");
				app.router.load('modal', {
					title : Template7.global.lbl.ERROR,
					message : Template7.global.lbl.CONFIGURATION_NETWORK_CASE_ERROR + " <b>" + rete + "</b>.<br/>" + Template7.global.lbl.CONFIGURATION_NETWORK_CANNOT_FIND + " " + rete + " " + Template7.global.lbl.CONFIGURATION_NETWORK_CONNECTION_WORKED_BUT + ".<br/>" + Template7.global.lbl.CONFIGURATION_NETWORK_TRY_TO_CONNECT + " <b>" + essid + "</b> " + Template7.global.lbl.CONFIGURATION_NETWOR_SEARCH + ".",
					buttons : {
						reload : true,
						devices : true
					}
				});
			}
		}
	}

	function sendNetworkConfig(essid, connected, password) {
		app.utils.ajaxRequest("POST", app.utils.GetRisorsa("network_set"), {
			callback : function(data) {
				app.f7.showPreloader(Template7.global.lbl.CONNECTING);
				setTimeout(function() {
					var container = app.$$('.modal-preloader');
					if (container.children('.progressbar, .progressbar-infinite').length) {
					} else {
						app.f7.showProgressbar(container, 0, 'blue');
					}

					sendNetworkConfigRecursive(connected, false, 0, 5, 10000, essid);
				}, 5000);
			},
			waitRequest : true,
			data : {
				essid : essid,
				pwd : password,
				dhcp : 1
			}
		});
	}

	function setNetwork() {
		if($(this).parent().hasClass("li-disabled")) {
			// Non faccio nulla, rete non utilizzabile
			app.f7.alert(Template7.global.lbl.NETWORK_NOT_USABLE_EXPLANATION);
			return;
		}
		
		var essid = $(this).attr("k_net"), connected = $(this).find(".connected"), password = $(this).attr("k_password");

		if (essid != "") {
			if (password == "true") {
				app.f7.modal({
					title : Template7.global.lbl.INSERT_WIFI_PASSWORD,
					afterText : '<div class="input-field"><input type="password" id="password" class="modal-text-input"></div>',
					buttons : [ {
						text : Template7.global.lbl.CANCEL
					}, {
						text : Template7.global.lbl.MODAL_BUTTON_OK,
						bold : true,
						onClick : function() {
							app.utils.keepAwake();

							var new_password = $('#password').val();
							sendNetworkConfig(essid, connected, new_password);
						}
					} ]
				});

				$('#password').hideShowPassword({
					show : 'infer',
					innerToggle : 'focus',
					states : {
						shown : {
							className : 'hideShowPassword-shown',
							changeEvent : 'passwordShown',
							props : {
								type : 'text'
							},
							toggle : {
								className : 'hideShowPassword-toggle-hide',
								content : Template7.global.lbl.HIDE,
								attr : {
									'aria-pressed' : 'true',
									title : Template7.global.lbl.HIDE_PASSWORD
								}
							}
						},
						hidden : {
							className : 'hideShowPassword-hidden',
							changeEvent : 'passwordHidden',
							props : {
								type : 'password'
							},
							toggle : {
								className : 'hideShowPassword-toggle-show',
								content : Template7.global.lbl.SHOW,
								attr : {
									'aria-pressed' : 'false',
									title : Template7.global.lbl.SHOW_PASSWORD,
								}
							}
						}
					}
				}).on('passwordShown', function() {
					app.utils.focusField('password');
				}).on('passwordHidden', function() {
					app.utils.focusField('password');
				});

				setTimeout(function() {
					app.utils.focusField('password');
				}, 500);
			} else {
				// Inibisco la sleep mode del cellulare
				app.utils.keepAwake();

				sendNetworkConfig(essid, connected, "");
			}
		}
	}

	return {
		init : init
	};
});