define([ "app", "js/models/deviceModel", "text!js/setup/prerender.html", "js/setup/setupView" ], function(app, DeviceModel, Template, View) {
	var bindings = [];
	var device = {};

	function init(parameters) {
		app.f7.showIndicator();

		// $('.screensaver').hide();

		app.utils.Log("INIT setupController");

		device = parameters.device;

		var compiled_template = Template7.compile(Template);
		var html = '<div class="popup">' + compiled_template({}) + '</div>';

		app.mainView.router.loadPage({
			content : html
		});

		View.render({
			bindings : bindings
		});

		app.utils.googleAnalytics('send', 'pageview', {
			'page' : 'setup'
		});

		app.f7.hideIndicator();
		app.f7.hidePreloader();

		jQuery.validator.addMethod("alphanumeric", function(value, element) {
			var regex = new RegExp("^([a-zA-Z0-9-_. ]{1,})$");
			return this.optional(element) || regex.test(value);
		}, Template7.global.lbl.ONLY_ALPHABETIC_AND_DIGITS);
		
		jQuery.validator.addMethod("regex", function(value, element, regexp) {
		    var re = new RegExp(regexp);
		    return this.optional(element) || re.test(value);
		}, Template7.global.lbl.PLEASE_CHECK_YOUR_INPUT);

		$('#serial').on('input', function() {
			$(this).val($(this).val().trim());
		});
		
		// Submit form
		$('#setup').validate({
			rules : {
				serial : {
					required : true,
					alphanumeric : true,
					maxlength : 9,
					minlength : 9,
					regex : "^(IN.[0-9]{1,})$"
				},
				name : {
					required : true,
					alphanumeric : true
				},
				pwd_1 : {
					required : function(element) {
						return $("#pwd_1").val() != "";
					},
					alphanumeric : true,
					maxlength : 19,
					minlength : 8
				},
				pwd_2 : {
					required : function(element) {
						return $("#pwd_1").val() != "";
					},
					equalTo : '#pwd_1',
					alphanumeric : true,
					maxlength : 19,
					minlength : 8
				}
			},
			messages : {
				serial : {
					alphanumeric : Template7.global.lbl.ONLY_ALPHABETIC_AND_DIGITS,
					minlength : Template7.global.lbl.SERIAL_LENGTH_RANGE,
					maxlength : Template7.global.lbl.SERIAL_LENGTH_RANGE,
					regex : Template7.global.lbl.SERIAL_FORMAT
				},
				name : {
					alphanumeric : Template7.global.lbl.ONLY_ALPHABETIC_AND_DIGITS
				},
				pwd_1 : {
					alphanumeric : Template7.global.lbl.ONLY_ALPHABETIC_AND_DIGITS,
					minlength : Template7.global.lbl.PASSWORD_LENGTH_RANGE,
					maxlength : Template7.global.lbl.PASSWORD_LENGTH_RANGE
				},
				pwd_2 : {
					required : Template7.global.lbl.PASSWORD_EQUAL_TO,
					equalTo : Template7.global.lbl.PASSWORD_EQUAL_TO
				}
			},
			submitHandler : function(form) {
				setSetup();
				return false;
			}
		});

		$('#pwd_1').hideShowPassword({
			show : 'infer',
			innerToggle : 'focus',
			states : {
				shown : {
					className : 'hideShowPassword-shown',
					changeEvent : 'passwordShown',
					props : {
						type : 'text'
					},
					toggle : {
						className : 'hideShowPassword-toggle-hide',
						content : Template7.global.lbl.HIDE,
						attr : {
							'aria-pressed' : 'true',
							title : Template7.global.lbl.HIDE_PASSWORD
						}
					}
				},
				hidden : {
					className : 'hideShowPassword-hidden',
					changeEvent : 'passwordHidden',
					props : {
						type : 'password'
					},
					toggle : {
						className : 'hideShowPassword-toggle-show',
						content : Template7.global.lbl.SHOW,
						attr : {
							'aria-pressed' : 'false',
							title : Template7.global.lbl.SHOW_PASSWORD,
						}
					}
				}
			}
		}).on('passwordShown', function() {
			app.utils.focusField('pwd_1');
		}).on('passwordHidden', function() {
			app.utils.focusField('pwd_1');
		});

		$('#pwd_2').hideShowPassword({
			show : 'infer',
			innerToggle : 'focus',
			states : {
				shown : {
					className : 'hideShowPassword-shown',
					changeEvent : 'passwordShown',
					props : {
						type : 'text'
					},
					toggle : {
						className : 'hideShowPassword-toggle-hide',
						content : Template7.global.lbl.HIDE,
						attr : {
							'aria-pressed' : 'true',
							title : Template7.global.lbl.HIDE_PASSWORD
						}
					}
				},
				hidden : {
					className : 'hideShowPassword-hidden',
					changeEvent : 'passwordHidden',
					props : {
						type : 'password'
					},
					toggle : {
						className : 'hideShowPassword-toggle-show',
						content : Template7.global.lbl.SHOW,
						attr : {
							'aria-pressed' : 'false',
							title : Template7.global.lbl.SHOW_PASSWORD,
						}
					}
				}
			}
		}).on('passwordShown', function() {
			app.utils.focusField('pwd_2');
		}).on('passwordHidden', function() {
			app.utils.focusField('pwd_2');
		});

		setTimeout(function() {
			app.utils.focusField('name_product');
		}, 1000);
	}

	function setSetup() {
		var name = app.$$('input[name=name]').val(), serial = app.$$('input[name=serial]').val(), pwd = app.$$('input[name=pwd_1]').val();

		serial = serial.trim();
		name = name.trim();
		pwd = pwd.trim();

		if (name == "" || serial == "") {
			app.f7.addNotification({
				title : Template7.global.lbl.FIELD_MISSING,
				hold : 2000
			});
		}

		app.utils.ajaxRequest("POST", app.utils.GetRisorsa("setup"), {
			callback : function(data) {
				if (data.success == true) {
					var device = app.utils.getDeviceByUID(data.UID);
					if (device !== false) {
						// E' già presente
						device.setName(name);
						device.setSerial(serial);
						device.setPassword(pwd);
						
						// Forzo lo standalone a true in caso il device fosse già presente
						device.setStandalone(true);
					} else {
						device = new DeviceModel();

						device.setName(data.setup.name);
						device.setSerial(data.setup.serial);
						device.setPassword(pwd);
						device.setUID(data.UID);
						device.setStandalone(true);
					}
					
					// Imposto il dispositivo come preferito se non ne ho nessuno di configurato
					var devices = app.utils.getDevices();
					if(devices !== false && devices.length > 0) {
						device.setPreferred(false);
					} else {
						device.setPreferred(true);
					}

					// Aggiungo/Aggiorno il dispositivo all'elenco dei
					// dispositivi
					device.save(app);

					// Imposto come dispositivo corrente
					app.utils.setCurrentDevice(device);

					app.f7.showPreloader(Template7.global.lbl.CONFIGURATION_IN_PROGRESS);
					setTimeout(function() {
						app.utils.networkConnection({
							SSID : Template7.global.ESSID_default + "_" + name,
							password : pwd,
							app : app
						});
					}, 10000);
				} else {
					app.f7.addNotification({
						title : Template7.global.lbl.ERROR + ": " + data.error,
						hold : 2000
					});
				}
			},
			data : {
				serial : serial,
				name : name,
				pwd : pwd
			}
		});
	}

	return {
		init : init
	};
});